﻿--[[ $Id: PlayerMenu.lua 67827 2008-04-03 07:53:08Z hshh $ ]]--
-- ACE2 init
local L = AceLibrary("AceLocale-2.2"):new("PlayerMenu")
local bs = LibStub("LibBabble-Spell-3.0")
local bst = LibStub("LibBabble-Spell-3.0"):GetLookupTable()
PlayerMenu = AceLibrary("AceAddon-2.0"):new("AceDB-2.0", "AceEvent-2.0", "AceHook-2.1", "AceConsole-2.0")
local Version = "3.1"
local _G = getfenv(0)

local SpellList = {}
SpellList["PRIEST"]={
	"Power Word: Fortitude",
	"Prayer of Fortitude",
	"Power Word: Shield",
	"Divine Spirit",
	"Prayer of Spirit",
	"Shadow Protection",
	"Prayer of Shadow Protection",
	"Power Infusion",
	"Cure Disease",
	"Abolish Disease",
	"Dispel Magic",
	"Resurrection",
}
SpellList["DRUID"]={
	"Gift of the Wild",
	"Mark of the Wild",
	"Thorns",
	"Remove Curse",
	"Abolish Poison",
	"Innervate",
	"Rebirth",
}
SpellList["PALADIN"]={
	"Redemption",
	"Blessing of Freedom",
	"Blessing of Kings",
	"Blessing of Light",
	"Blessing of Might",
	"Blessing of Protection",
	"Blessing of Sacrifice",
	"Blessing of Salvation",
	"Blessing of Sanctuary",
	"Blessing of Wisdom",
	"Greater Blessing of Kings",
	"Greater Blessing of Light",
	"Greater Blessing of Might",
	"Greater Blessing of Salvation",
	"Greater Blessing of Sanctuary",
	"Greater Blessing of Wisdom",
	"Divine Intervention",
	"Cleanse",
}
SpellList["WARLOCK"]={
	"Unending Breath",
	"Ritual of Summoning",
}
SpellList["MAGE"]={
	"Arcane Intellect",
	"Arcane Brilliance",
	"Dampen Magic",
	"Amplify Magic",
	"Remove Lesser Curse",
}
SpellList["SHAMAN"]={
	"Cure Poison",
	"Water Breathing",
	"Cure Disease",
}
local SpellIgnoreClearTarget = {
	"Ritual of Summoning",
}

function PlayerMenu:OnInitialize()
	-- prepare to save blizzard origin variables
	self.UnitPopupMenus={FRIEND={}, PLAYER={}, PARTY={}, RAID_PLAYER={}}
	
	-- included preparing Order_Orig, Button_Orig
	self.Order_Orig = {}
	self.Button_Orig = {}
	for k,v in pairs(_G.UnitPopupMenus["FRIEND"]) do
		self.UnitPopupMenus["FRIEND"][k]=v
		self.Order_Orig[k]=v
		self.Button_Orig[v]=1
	end
	tinsert(self.Order_Orig, getn(self.Order_Orig)-1, "GUILD_INVITE")
	tinsert(self.Order_Orig, getn(self.Order_Orig)-1, "ADD_FRIEND")
	tinsert(self.Order_Orig, getn(self.Order_Orig)-1, "GET_NAME")
	tinsert(self.Order_Orig, getn(self.Order_Orig)-1, "WHO")
	tinsert(self.Order_Orig, getn(self.Order_Orig)-1, "EXTMENU")
	self.Button_Orig["GUILD_INVITE"]=1
	self.Button_Orig["ADD_FRIEND"]=1
	self.Button_Orig["GET_NAME"]=1
	self.Button_Orig["WHO"]=1
	self.Button_Orig["EXTMENU"]=1
	self.Menus=getn(self.Order_Orig)
	
	for k,v in pairs(_G.UnitPopupMenus["PLAYER"]) do
		self.UnitPopupMenus["PLAYER"][k]=v
	end
	for k,v in pairs(_G.UnitPopupMenus["PARTY"]) do
		self.UnitPopupMenus["PARTY"][k]=v
	end
	for k,v in pairs(_G.UnitPopupMenus["RAID_PLAYER"]) do
		self.UnitPopupMenus["RAID_PLAYER"][k]=v
	end

	-- register saved variables
	self:RegisterDB("PlayerMenu_Settings")
	-- init default variables
	self:RegisterDefaults("profile", {
		leftButton = false,
		order = self.Order_Orig,
		button = self.Button_Orig,
		extAutoHide = 3,
		useFocus = false,
		clearTargetAfterCast = true,
	})
	-- found old saved variables, reset it.
	if (self.db.profile.version ~= Version) then
		self:ResetDB("profile")
		self:Print(L["MSG_RESETED"])
		self.db.profile.version = Version
	end

	-- register console slash commands
	self:RegisterChatCommand(
		{ "/playermenu", "/pm", "/apm" },
		{
			type = "group",
			args = {
				left_button = {
					order = 100,
					type = "toggle",
					name = L["Left Button Menu"],
					desc = L["leftButton_desc"],
					get = function() return self.db.profile.leftButton end,
					set = function(t)
						self.db.profile.leftButton = t
						if (self.db.profile.leftButton) then
							self:Print(L["MSG_LEFT_ON"])
						else
							self:Print(L["MSG_LEFT_OFF"])
						end
						self:Reload()
					end,
				},
				usefocus = {
					order = 200,
					type = "toggle",
					name = L["Use Focus to Cast"],
					desc = L["Use focus feature to cast spell instead of using target."],
					get = function() return self.db.profile.useFocus end,
					set = function(t) self.db.profile.useFocus = t end,
				},
				hide_delay = {
					order = 300,
					type = "range",
					name = L["Extend Menu Hide Delay"],
					min = 1,
					max = 20,
					step = 1,
					desc = L["Set extend menu auto hide delay time."],
					get = function() return self.db.profile.extAutoHide end,
					set = function(r)
						self.db.profile.extAutoHide=r
					end
				},
				menu_order = {
					order = 400,
					type = "text",
					name = L["Menu Order"],
					desc = L["order_desc"],
					usage = L["order_usage"],
					get = function()
						local order=""
						for k,v in pairs(self.db.profile.order) do
							if (self.db.profile.button[v]==1) then
								order=order..v..":1,"
							else
								order=order..v..":0,"
							end
						end
						order = string.sub(order, 1, -2)
						return order
					end,
					set = function (c)
						c = string.upper(c)
						local newOrder = {}
						local newButton = {}
						local i = 1
						for b,e in pairs(string.gmatch(c, "([%w_]+):(%d)")) do
							e = tonumber(e)
							if (e~=1 and e~=0) then
								e=1
							end
							if (b~=nil and self.Button_Orig[b]~=nil) then
								newOrder[i]=b
								newButton[b]=e
								i=i+1
							end
						end
						if (table.getn(newOrder)==self.Menus) then
							self.db.profile.order = newOrder
							self.db.profile.button = newButton
							self:Reload()
						end
					end,
					message = L["MSG_MENU_ORDER_CHANGED"]
				},
				clear_target = {
					order = 500,
					type = "toggle",
					name = L["Clear Target After Cast"],
					desc = L["Clear Target After Cast"],
					get = function() return self.db.profile.clearTargetAfterCast end,
					set = function(t) self.db.profile.clearTargetAfterCast = t end,
				},
				reset = {
					order = 1000,
					type = "execute",
					name = L["Reset"],
					desc = L["reset_desc"],
					func = function()
						self:ResetDB("profile")
						self:Reload()
						self:Print(L["MSG_RESETED"])
					end
				}
			}
		}
	)

	--init for extended menu
	local _, class = UnitClass("player")
	self.SpellValid = {}
	self.SpellValid_Count = 0
	if SpellList[class] then
		for _,v in ipairs(SpellList[class]) do
			self.SpellValid[bst[v]]=bs:GetSpellIcon(v)
			self.SpellValid_Count = self.SpellValid_Count + 1
		end
	end
	self.SpellIgnoreClearTarget = {}
	for _,v in ipairs(SpellIgnoreClearTarget) do
		self.SpellIgnoreClearTarget[bst[v]]=1
	end
	self.ExtMaxButton = 25
end

function PlayerMenu:OnEnable()
	self:InitSpell()
	self:RegisterEvent("SPELLS_CHANGED", "InitSpell")
	self:RegisterEvent("LEARNED_SPELL_IN_TAB", "InitSpell")
	self:RegisterEvent("PLAYER_REGEN_DISABLED", "ExtMenu_AutoHide") --in battle

	self:SecureHook("UnitPopup_OnClick")
	self:SecureHook("UnitPopup_HideButtons")
	if (self.db.profile.leftButton) then
		self:SecureHook("SetItemRef")
	end

	UnitPopupButtons["ADD_FRIEND"] = { text = TEXT(ADD_FRIEND), dist = 0 }
	UnitPopupButtons["GUILD_INVITE"] = { text = L["TEXT_GUILD_INVITE"], dist = 0 }
	UnitPopupButtons["GET_NAME"] = { text = L["TEXT_GET_NAME"], dist = 0 }
	UnitPopupButtons["WHO"] = { text = TEXT(WHO), dist = 0 }
	UnitPopupButtons["EXTMENU"] = { text = L["Extended Menu"], dist = 0 }

	UnitPopupMenus["FRIEND"] = {}
	for k,v in pairs(self.db.profile.order) do
		if (self.db.profile.button[v]==1) then
			UnitPopupMenus["FRIEND"][k]=v
		end
	end

	tinsert(UnitPopupMenus["PLAYER"], getn(UnitPopupMenus["PLAYER"])-1, "ADD_FRIEND")
	tinsert(UnitPopupMenus["PLAYER"], getn(UnitPopupMenus["PLAYER"])-1, "GET_NAME")
	tinsert(UnitPopupMenus["PLAYER"], getn(UnitPopupMenus["PLAYER"])-1, "WHO")
	tinsert(UnitPopupMenus["PLAYER"], getn(UnitPopupMenus["PLAYER"])-1, "EXTMENU")

	tinsert(UnitPopupMenus["PARTY"], getn(UnitPopupMenus["PARTY"])-1, "ADD_FRIEND")
	tinsert(UnitPopupMenus["PARTY"], getn(UnitPopupMenus["PARTY"])-1, "GET_NAME")
	tinsert(UnitPopupMenus["PARTY"], getn(UnitPopupMenus["PARTY"])-1, "WHO")
	tinsert(UnitPopupMenus["PARTY"], getn(UnitPopupMenus["PARTY"])-1, "EXTMENU")

	tinsert(UnitPopupMenus["RAID_PLAYER"], getn(UnitPopupMenus["RAID_PLAYER"])-1, "ADD_FRIEND")
	tinsert(UnitPopupMenus["RAID_PLAYER"], getn(UnitPopupMenus["RAID_PLAYER"])-1, "GET_NAME")
	tinsert(UnitPopupMenus["RAID_PLAYER"], getn(UnitPopupMenus["RAID_PLAYER"])-1, "WHO")
	tinsert(UnitPopupMenus["RAID_PLAYER"], getn(UnitPopupMenus["RAID_PLAYER"])-1, "EXTMENU")
end

function PlayerMenu:OnDisable()
	UnitPopupButtons["ADD_FRIEND"] = nil
	UnitPopupButtons["GUILD_INVITE"] = nil
	UnitPopupButtons["GET_NAME"] = nil
	UnitPopupButtons["WHO"] = nil
	UnitPopupButtons["EXTMENU"] = nil

	UnitPopupMenus["FRIEND"]={}
	UnitPopupMenus["PLAYER"]={}
	UnitPopupMenus["PARTY"]={}
	UnitPopupMenus["RAID_PLAYER"]={}
	for k,v in pairs(self.UnitPopupMenus["FRIEND"]) do
		UnitPopupMenus["FRIEND"][k]=v
	end
	for k,v in pairs(self.UnitPopupMenus["PLAYER"]) do
		UnitPopupMenus["PLAYER"][k]=v
	end
	for k,v in pairs(self.UnitPopupMenus["PARTY"]) do
		UnitPopupMenus["PARTY"][k]=v
	end
	for k,v in pairs(self.UnitPopupMenus["RAID_PLAYER"]) do
		UnitPopupMenus["RAID_PLAYER"][k]=v
	end

	self:UnhookAll()
	PMEM_Frame:Hide()
end

function PlayerMenu:UnitPopup_HideButtons()
	-- hide ext menu
	self:ExtMenu_AutoHide()

	local dropdownMenu = getglobal(UIDROPDOWNMENU_INIT_MENU)
	local diffFactionGroup
	if dropdownMenu.unit and UnitFactionGroup("player") ~= UnitFactionGroup(dropdownMenu.unit) then
		diffFactionGroup = true
	end
	local realm
	if dropdownMenu.unit then
		_,realm = UnitName(dropdownMenu.unit)
	end

	for index, value in ipairs(UnitPopupMenus[dropdownMenu.which]) do
		if value == "GET_NAME" then
			if dropdownMenu.name == UnitName("player") then
				UnitPopupShown[UIDROPDOWNMENU_MENU_LEVEL][index] = 0
			else
				UnitPopupShown[UIDROPDOWNMENU_MENU_LEVEL][index] = 1
			end
		elseif value == "WHO" or value == "ADD_FRIEND" then
			if dropdownMenu.name == UnitName("player") or (realm and realm ~="") or diffFactionGroup then
				UnitPopupShown[UIDROPDOWNMENU_MENU_LEVEL][index] = 0
			else
				UnitPopupShown[UIDROPDOWNMENU_MENU_LEVEL][index] = 1
			end
		elseif value == "GUILD_INVITE" then
			if not CanGuildInvite() or dropdownMenu.name == UnitName("player") or (realm and realm ~="") or diffFactionGroup then
				UnitPopupShown[UIDROPDOWNMENU_MENU_LEVEL][index] = 0
			else
				UnitPopupShown[UIDROPDOWNMENU_MENU_LEVEL][index] = 1
			end
		elseif value == "EXTMENU" then
			UnitPopupShown[UIDROPDOWNMENU_MENU_LEVEL][index] = 1
		end
	end
end

function PlayerMenu:UnitPopup_OnClick()
	local dropdownFrame = getglobal(UIDROPDOWNMENU_INIT_MENU)
	local button = this.value
	local name = dropdownFrame.name

	if (button == "ADD_FRIEND") then
		AddFriend(name)
	elseif (button == "GUILD_INVITE") then
		GuildInvite(name)
	elseif (button == "GET_NAME") then
		ChatFrameEditBox:Show()
		local realm
		if dropdownFrame.unit then
			_,realm = UnitName(dropdownFrame.unit)
		end
		if (realm and realm ~="") then
			ChatFrameEditBox:Insert(name .. " - " .. realm)
		else
			ChatFrameEditBox:Insert(name)
		end
	elseif (button == "WHO") then
		SendWho(name)
	elseif (button == "EXTMENU") then
		self:ExtMenu_InitButton(name)
		self:ExtMenu_Show()
	end
	PlaySound("UChatScrollButton")
end

function PlayerMenu:SetItemRef(link, text, button)
	if ( strsub(link, 1, 6) == "player" ) then
		local namelink = strsub(link, 8);
		local name, lineid = strsplit(":", namelink);
		if ( name and (strlen(name) > 0) ) then
			if ( IsModifiedClick("CHATLINK") ) then
				local staticPopup;
				staticPopup = StaticPopup_Visible("ADD_IGNORE");
				if ( staticPopup ) then
					-- If add ignore dialog is up then enter the name into the editbox
					getglobal(staticPopup.."EditBox"):SetText(name);
					return;
				end
				staticPopup = StaticPopup_Visible("ADD_MUTE");
				if ( staticPopup ) then
					-- If add ignore dialog is up then enter the name into the editbox
					getglobal(staticPopup.."EditBox"):SetText(name);
					return;
				end
				staticPopup = StaticPopup_Visible("ADD_FRIEND");
				if ( staticPopup ) then
					-- If add ignore dialog is up then enter the name into the editbox
					getglobal(staticPopup.."EditBox"):SetText(name);
					return;
				end
				staticPopup = StaticPopup_Visible("ADD_GUILDMEMBER");
				if ( staticPopup ) then
					-- If add ignore dialog is up then enter the name into the editbox
					getglobal(staticPopup.."EditBox"):SetText(name);
					return;
				end
				staticPopup = StaticPopup_Visible("ADD_TEAMMEMBER");
				if ( staticPopup ) then
					-- If add ignore dialog is up then enter the name into the editbox
					getglobal(staticPopup.."EditBox"):SetText(name);
					return;
				end
				staticPopup = StaticPopup_Visible("ADD_RAIDMEMBER");
				if ( staticPopup ) then
					-- If add ignore dialog is up then enter the name into the editbox
					getglobal(staticPopup.."EditBox"):SetText(name);
					return;
				end
				staticPopup = StaticPopup_Visible("CHANNEL_INVITE");
				if ( staticPopup ) then
					getglobal(staticPopup.."EditBox"):SetText(name);
					return;
				end
				if ( ChatFrameEditBox:IsVisible() ) then
					ChatFrameEditBox:Insert(name);
				elseif ( HelpFrameOpenTicketText:IsVisible() ) then
					HelpFrameOpenTicketText:Insert(name);
				else
					SendWho("n-"..name);					
				end
				
			else
				ChatFrameEditBox:Hide()
				FriendsFrame_ShowDropdown(name, 1, lineid);
			end
		end
		return;
	end
end

function PlayerMenu:Reload()
	self:OnDisable()
	self:OnEnable()
end

function PlayerMenu:InitSpell()
	self.SpellCache = {}
	self.SpellCache_Count = 0

	if self.SpellValid_Count == 0 then
		return
	end

	local numTotalSpells = 0
	for i=1, MAX_SKILLLINE_TABS do
		local name, _, _, numSpells = GetSpellTabInfo(i)
		if name then
			numTotalSpells = numTotalSpells + numSpells
		end
	end

	local SpellExists = {}
	for i=1, numTotalSpells do
		local spellName = GetSpellName(i, BOOKTYPE_SPELL)
		if spellName and self.SpellValid[spellName] and not SpellExists[spellName] then
			table.insert(self.SpellCache, spellName)
			SpellExists[spellName] = 1
		end
	end
	self.SpellCache_Count = getn(self.SpellCache)
end
function PlayerMenu:ExtMenu_ButtonOnClick()
	PMEM_Frame:Hide()
end
function PlayerMenu:ExtMenu_InitButton(name)
	if not name then
		name = UnitName("player")
	end
	PMEM_Frame_Button1:SetText(L["Set Target"])
	PMEM_Frame_Button1:SetAttribute("type1", "macro")
	PMEM_Frame_Button1:SetAttribute("macrotext", "/target "..name)
	PMEM_Frame_Button1:Show()
	PMEM_Frame_Button2:SetText(L["Set Focus"])
	PMEM_Frame_Button2:SetAttribute("type1", "macro")
	PMEM_Frame_Button2:SetAttribute("macrotext", "/focus "..name)
	PMEM_Frame_Button2:Show()
	PMEM_Frame_Button3:SetText(TRADE)
	PMEM_Frame_Button3:SetAttribute("type1", "macro")
	PMEM_Frame_Button3:SetAttribute("macrotext", "/target "..name.."\n/trade")
	PMEM_Frame_Button3:Show()

	local spell_id = 1
	for i=4, self.SpellCache_Count+3 do
		local button = getglobal("PMEM_Frame_Button"..i)
		button:SetText(self.SpellCache[spell_id])
		button:SetAttribute("type1", "macro")
		if self.db.profile.useFocus then
			button:SetAttribute("macrotext", "/focus "..name.."\n/cast [target=focus] "..self.SpellCache[spell_id].."\n/stopcasting\n/clearfocus")
		else
			local lastTarget = UnitName("target")
			if lastTarget and self.db.profile.clearTargetAfterCast and not self.SpellIgnoreClearTarget[self.SpellCache[spell_id]] then
				button:SetAttribute("macrotext", "/target "..name.."\n/cast "..self.SpellCache[spell_id].."\n/stopcasting\n/target "..lastTarget)
			else
				button:SetAttribute("macrotext", "/target "..name.."\n/cast "..self.SpellCache[spell_id].."\n/stopcasting\n")
			end
		end
		button:Show()
		spell_id = spell_id + 1
	end

	for i=self.SpellCache_Count+4,self.ExtMaxButton do
		local button = getglobal("PMEM_Frame_Button"..i)
		button:Hide()
	end

	self.ExtMenuWidth = 0
	for i=1, self.ExtMaxButton do
		local button = getglobal("PMEM_Frame_Button"..i)
		local width = button:GetTextWidth()
		if width > self.ExtMenuWidth then
			self.ExtMenuWidth = width
		end
	end
	self.ExtMenuWidth = self.ExtMenuWidth + 2*12
	self.ExtMenuHeight = (self.SpellCache_Count + 3 + 2) * 12 -- 3 fix button, top and bottom space
end
function PlayerMenu:ExtMenu_AutoHide()
	if PlayerMenu:IsEventScheduled("PlayerMenu_ExtMenu_AutoHide") then
		PlayerMenu:CancelScheduledEvent("PlayerMenu_ExtMenu_AutoHide")
	end
	if PMEM_Frame:IsVisible() then
		PMEM_Frame:Hide()
	end
end
function PlayerMenu:ExtMenu_Show()
	self:ScheduleRepeatingEvent("PlayerMenu_ExtMenu_AutoHide", self.ExtMenu_AutoHide, self.db.profile.extAutoHide)
	local point, relativeTo, relativePoint, xOfs, yOfs = this:GetParent():GetPoint()
	PMEM_Frame:SetHeight(self.ExtMenuHeight)
	PMEM_Frame:SetWidth(self.ExtMenuWidth)
	PMEM_Frame:ClearAllPoints()
	PMEM_Frame:SetPoint(point, relativeTo, relativePoint, xOfs, yOfs)
	PMEM_Frame:Show()
end
function PlayerMenu:ExtMenu_OnEnter(button)
	if self:IsEventScheduled("PlayerMenu_ExtMenu_AutoHide") then
		self:CancelScheduledEvent("PlayerMenu_ExtMenu_AutoHide")
	end
	if button then
		local spell = this:GetText()
		if spell and spell ~= L["Set Target"] and spell ~= L["Set Focus"] and spell ~= TRADE then
			PMEM_FrameSpellIcon:SetTexture(self.SpellValid[spell])
			PMEM_FrameSpellIcon:Show()
		end
	end
end
function PlayerMenu:ExtMenu_OnLeave()
	if not self:IsEventScheduled("PlayerMenu_ExtMenu_AutoHide") then
		self:ScheduleRepeatingEvent("PlayerMenu_ExtMenu_AutoHide", self.ExtMenu_AutoHide, self.db.profile.extAutoHide)
	end
	PMEM_FrameSpellIcon:Hide()
end
